#!/usr/bin/python


import array
import os
import commands
import time
import math
import sys
import string
import fileinput
import re
import glob

# ---------------------------------------------------------------------------
# IMPORTANT PARAMETERS
# ---------------------------------------------------------------------------
#todo - try using os.path.join
HOME            = os.environ['HOME'] + "/"
COIN_DIR        = HOME + "COIN/coin-Decomp/"
COIN_DATA_DIR   = COIN_DIR + "Decomp/data/"
RUN_DIR         = HOME + "running/decomp/"

INST_SET        = "tsplib"
DATA_DIR        = COIN_DATA_DIR + "TSP/TSPLIB/"
#BEST_BOUND_FILE = DATA_DIR + "ap3.opt"
INST_LIST       = DATA_DIR + "tsplib.list"

OUTPUT_DIR1     = RUN_DIR + "tsp/tspc/"
OUTPUT_DIR2     = RUN_DIR + "tsp/tspp/"

INFTY=10e10
ABS_GAPTOL=0.1
EPS_TIME = 0.01
#TIME_LIMIT = 1800
delim = "  "
# ---------------------------------------------------------------------------
# ---------------------------------------------------------------------------



def find_float(arr0,st0,fl0):
	for line in arr0:
		find = re.search(st0,line)
		if (find >= 0):
			fl0 = float(re.search('-*[0-9]+\.[0-9]+',line).group())
			return 1,fl0
	return -1,fl0

def find_float_1(arr0,st0,fl0):
	for line in arr0:
		find = re.search(st0,line)
		if (find >= 0):
			find = re.search('-*[0-9]+\.[0-9]+',line)
			if (find>=0):
				l = re.findall('-*[0-9]+\.[0-9]+',line)
				if (len(l)<3):
					return -1,fl0
				else:
					fl0 = float(l[2])
					return 1,fl0
			else:
				return -1,fl0
	return -1,fl0

def find_float_e(arr0,st0,fl0):
	for line in arr0:
		find = re.search(st0,line)
		if (find >= 0):
			fl0 = float(re.search('-*[0-9]+\.[0-9]*e*\+*[0-9]*',line).group())
			return 1,fl0
	return -1,fl0

def find_int(arr0,st0,in0):
	in0=0
	for line in arr0:
		find = re.search(st0,line)
		if (find >= 0):
			in0 = int(re.search('-*[0-9]+$',line).group())
			return 1,in0
	return -1,in0

# ---------------------------------------------------------------------------
# ---------------------------------------------------------------------------

for i in range(0,len(sys.argv)):
	if (sys.argv[i]=='-d'):
		if (i==len(sys.argv)):
			print "Missing argument to '-d'"
			print "usage: python report_decomp.py [-d <path to dir>]"
			sys.exit(0)
		else:
			OUTPUT_DIR=sys.argv[i+1]
			break

a = []
fl0 = 0.0
in0 = 0

flist=open(INST_LIST,'r')
a=flist.read().split()
flist.close()
a.sort()

print "### Instance set:", INST_SET



#print "##%9s"%"Instance",delim,"%12s"%"CPX Obj",delim,"%7s"%"miptime",delim,"%12s"%"tkegraph",delim,"%8s"%"Time",delim,"%6s"%"binit","%8s"%"cost-gap",delim,"%8s"%"time-LP",delim,"%8s"%"time-GS",delim,"%9s"%"time-cost",delim,"%6s"%"edges",delim,"%6s"%"constr"

#TODO - get cplex times... 

#check for errors first
print "### Reading from directory:", OUTPUT_DIR1
for instance in a:
        filewcard = OUTPUT_DIR1 + instance + ".condor.err.*"
        filename = glob.glob(filewcard)
        statinfo = os.stat(filename[0])
        if (statinfo.st_size):
                print "ERROR %10s"%instance

print "### Reading from directory:", OUTPUT_DIR1
for instance in a:
        filewcard = OUTPUT_DIR1 + instance + ".condor.err.*"
        filename = glob.glob(filewcard)[0]
        statinfo = os.stat(filename)
        if (statinfo.st_size):
                print "ERROR %10s" % instance
                fil        = open(filename, 'r')
                whole_file = fil.read().split('\n')
                fil.close()
                print whole_file



# print "### Reading from directory:", OUTPUT_DIR2
# for instance in a:
#         filewcard = OUTPUT_DIR2 + instance + ".condor.err.*"
#         filename = glob.glob(filewcard)[0]
#         statinfo = os.stat(filename)
#         if (statinfo.st_size):
#                 print "ERROR %10s" % instance
#                 fil        = open(filename, 'r')
#                 whole_file = fil.read().split('\n')
#                 fil.close()
#                 print whole_file













# for instance in a:
# 	print "%11s"%instance,

# 	fil=open(BEST_BOUND_FILE,'r')
# 	whole_file=fil.read().split('\n')
# 	fil.close()

# 	best_ub = INFTY
# 	find,best_ub=find_float(whole_file,instance,best_ub)
# 	print delim, "%12.1f"%best_ub,

# #TODO: check if file err file is size 0 


# #make this a function
# 	filename=OUTPUT_DIR1+"/"+INST_SET+"/"+instance+".condor.out"
# 	fil=open(filename,'r')
# 	whole_file=fil.read().split('\n')
# 	fil.close()
	
# 	bestObjValue = INFTY
# 	find,bestObjValue=find_float(whole_file,'flow cost =',bestObjValue)
# 	if (find<0 or bestObjValue >= INFTY):
# 		print delim, "%12s"%"NF",
# 	else:
# 		print delim, "%12.1f"%bestObjValue,

# 	solTime = INFTY
# 	find,solTime=find_float(whole_file,'time taken =',solTime)
# 	if (find<0 or solTime >= INFTY):
# 		print delim, "%8s"%"NF",
# 	else:
# 		if (solTime<EPS_TIME):
# 			solTime = EPS_TIME
# 		print delim, "%8.2f"%solTime,


# 	n = INFTY
# 	find,n = find_int(whole_file,'binary-search iterations =',n)
# 	if (find<0):
# 		print delim, "%6s"%"NF",
# 	else:
# 		print delim, "%6d"%n,

                

# 	"""
# 	flowCongestion = INFTY
# 	find,flowCongestion =find_float(whole_file,'flow congestion =',flowCongestion)
# 	if (find<0 or flowCongestion >= INFTY):
# 		print delim, "%8s"%"NF",
# 	else:
# 		print delim, "%8.2f"%flowCongestion,
# 	"""

# 	if (bestObjValue<INFTY and best_ub<INFTY):
# 		costFactor = abs(bestObjValue-best_ub)/bestObjValue*100,
# 		print delim, "%8.2f"%costFactor,
# 	else:
# 		print delim, "%8s"%"NF",

# 	timeInLP = INFTY
# 	find,timeInLP =find_float(whole_file,'time in LP solves =',timeInLP)
# 	if (find<0 or timeInLP >= INFTY):
# 		print delim, "%8s"%"NF",
# 	else:
# 		print delim, "%8.2f"%timeInLP,

# 	timeInGS = INFTY
# 	find,timeInGS =find_float(whole_file,'time in golden-section =',timeInGS)
# 	if (find<0 or timeInGS >= INFTY):
# 		print delim, "%8s"%"NF",
# 	else:
# 		print delim, "%8.2f"%timeInGS,

# 	timeInUF = INFTY
# 	find,timeInUF =find_float(whole_file,'time in update-cost =',timeInUF)
# 	if (find<0 or timeInUF >= INFTY):
# 		print delim, "%9s"%"NF",
# 	else:
# 		print delim, "%9.2f"%timeInUF,

# 	n = INFTY
# 	find,n = find_int(whole_file,'Number of edges              =',n)
# 	if (find<0):
# 		print delim, "%6s"%"NF",
# 	else:
# 		print delim, "%6d"%n,
	
# 	n = INFTY
# 	find,n = find_int(whole_file,'Number of mutual constraints =',n)
# 	if (find<0):
# 		print delim, "%6s"%"NF",
# 	else:
# 		print delim, "%6d"%n,

# 	print ''




import matplotlib.pyplot as plt

plt.rc('axes', grid=True)
plt.rc('grid', color='0.75', linestyle='-', linewidth=0.5)

#textsize = 9
#left, width = 0.1, 0.8
#rect1 = [left, 0.7, width, 0.2]
#rect2 = [left, 0.3, width, 0.4]
#rect3 = [left, 0.1, width, 0.2]


fig = plt.figure(facecolor='white')
axescolor  = '#f6f6f6'  # the axies background color

ax1 = fig.add_subplot(111)

#ax1 = fig.add_axes(rect1, axisbg=axescolor)  #left, bottom, width, height
#ax2 = fig.add_axes(rect2, axisbg=axescolor, sharex=ax1)
#ax2t = ax2.twinx()
#ax3  = fig.add_axes(rect3, axisbg=axescolor, sharex=ax1)

### plot the relative strength indicator
yUB =  (5, 4.5, 4.0, 2.0, 2.0)
yLB =  (1.0, 1.2, 1.4, 1.97, 1.99)
iter = (0.123, 0.324, 0.426,
        0.529, 0.6291)
yUB1 = (4.5, 4.4, 4.3, 2.4, 2.3, 1.3, 1.0)
yLB1 = (0.0, 0.1, -0.1, 0.9, 0.8, 0.8, 1.0)
iter1 = (0.01, 0.04, 0.08, 0.09, 0.10, 0.11, 0.123)

yUB2 = (4.4, 4.3, 2.4, 2.3, 1.2, 1.2, 1.2)
yLB2 = (0.1, 0.1, -0.1, 0.5, 0.4, 0.6, 1.2)
iter2 = (0.123, 0.124, 0.126, 0.22, 0.23, 0.31, 0.324)

fillcolor = 'darkgoldenrod'

ax1.plot(iter, yUB, color='b')
ax1.plot(iter, yLB, color='g')

ax1.plot(iter1, yUB1, color='r')
ax1.plot(iter1, yLB1, color='k')

ax1.plot(iter2, yUB2, color='r')
ax1.plot(iter2, yLB2, color='k')
#ax1.axhline(70, color=fillcolor)
#ax1.axhline(30, color=fillcolor)
#ax1.fill_between(r.date, rsi, 70, where=(rsi>=70), facecolor=fillcolor, edgecolor=fillcolor)
#ax1.fill_between(r.date, rsi, 30, where=(rsi<=30), facecolor=fillcolor, edgecolor=fillcolor)
#ax1.text(0.6, 0.9, '>70 = overbought', va='top', transform=ax1.transAxes, fontsize=textsize)
#ax1.text(0.6, 0.1, '<30 = oversold', transform=ax1.transAxes, fontsize=textsize)
#ax1.set_ylim(0, 6.0)
#ax1.set_yticks([2.0,7.0])
#ax1.text(0.025, 0.95, 'RSI (14)', va='top', transform=ax1.transAxes, fontsize=textsize)
#ax1.set_title('%s daily'%ticker)

# rotates and right aligns the x labels, and moves the bottom of the
# axes up to make room for them
fig.autofmt_xdate()


plt.show()



#!/usr/bin/env python

# Generalized Assignment Problem
# argument should be a problem file, see Dip/examples/GAP_Instance.cpp for format
# for an e.g. see gap0512-2.dat included in this directory

DEBUGGING = False

import sys

from pulp import *

try:
    import path
except ImportError:
    pass
        
if DEBUGGING:
    from dippy import dippy
else:
    import coinor.dippy as dippy

debug_print = False

tol = pow(pow(2, -24), 2.0 / 3.0)

# parse data file
input = open(sys.argv[1])

line = input.readline().split()
NUM_MACHINES = int(line[0])
NUM_TASKS = int(line[1])
MACHINES = range(NUM_MACHINES)
TASKS = range(NUM_TASKS)
MACHINES_TASKS = [(m, t) for m in MACHINES for t in TASKS]

COSTS = []
for m in MACHINES:
    line = input.readline().split()
    assert len(line) == NUM_TASKS
    COSTS.append([int(f) for f in line])

RESOURCE_USE = []
for m in MACHINES:
    line = input.readline().split()
    assert len(line) == NUM_TASKS
    RESOURCE_USE.append([int(f) for f in line])

line = input.readline().split()
assert len(line) == NUM_MACHINES
CAPACITIES = [int(f) for f in line]


assignVars = []
for m in MACHINES:
    v = []
    for t in TASKS:
        v.append(LpVariable("M%dT%d" % (m, t), cat=LpBinary))
    assignVars.append(v)

prob = dippy.DipProblem("GAP",
                        display_mode = 'xdot', 
                        layout = 'dot',
                        display_interval = None,
                        )

# objective
prob += lpSum(assignVars[m][t] * COSTS[m][t] for m, t in MACHINES_TASKS), "min"

# machine capacity (knapsacks, relaxation)
for m in MACHINES:
    prob.relaxation[m] += lpSum(assignVars[m][t] * RESOURCE_USE[m][t] for t in TASKS) <= CAPACITIES[m]

# assignment
for t in TASKS:
    prob += lpSum(assignVars[m][t] for m in MACHINES) == 1

def solve_subproblem(prob, machine, redCosts):
    if debug_print:
        print "solve_subproblem..."
        print redCosts
   
    # get tasks which have negative reduced costs
    task_idx = [t for t in TASKS if redCosts[assignVars[machine][t]] < 0]
    vars = [assignVars[machine][t] for t in task_idx]
    obj = [-redCosts[assignVars[machine][t]] for t in task_idx]
    weights = [RESOURCE_USE[machine][t] for t in task_idx]

    z, solution = knapsack01(obj, weights, CAPACITIES[machine])

    # Get the reduced cost of the knapsack solution and waste
    if debug_print:
        print [(v, redCosts[v]) for v in vars]
        print obj
        print "z, solution =", z, solution

    rc = -z

    if debug_print:
        print "rc", rc
    # Return the solution if the reduced cost is low enough
    # ...
    if rc > tol: # ... or an empty location is "useful"
       
        var_values = {}

        var_tuple = (0.0, 0.0, var_values)
        if debug_print:
            print "Empty solution is optimal"
            print var_tuple
        return [var_tuple]

    orig_cost = sum(prob.objective.get(vars[idx]) for idx in solution)
    var_values = dict([(vars[i], 1) for i in solution])

    var_tuple = (orig_cost, rc, var_values)
    rcCheck = 0.0
    for v in var_values.keys():
        rcCheck += redCosts[v] * var_values[v]
    if debug_print:
        print "Checking rc calc", rc, rcCheck 
        print var_tuple
    return [var_tuple]
    return None

def knapsack01(obj, weights, capacity):
    """ 0/1 knapsack solver, maximizes profit. weights and capacity integer """
        
    debug_subproblem = False
    
    assert len(obj) == len(weights)
    n = len(obj)
    if n == 0:
        return 0, []

    if (debug_subproblem):
        relaxation = pulp.LpProblem('relaxation', pulp.LpMaximize)
        relax_vars = [str(i) for i in range(n)]
        var_dict   = LpVariable.dicts("", relax_vars, 0, 1, LpBinary)
        relaxation += lpSum(var_dict[str(i)] * weights[i] for i in range(n)) <= capacity
        relaxation += lpSum(var_dict[str(i)] * obj[i] for i in range(n))
        relaxation.solve()
        relax_obj = value(relaxation.objective)

        solution =  [i for i in range(n) if var_dict[str(i)].varValue > tol ]

        print relax_obj, solution


    c = [[0]*(capacity+1) for i in range(n)]
    added = [[False]*(capacity+1) for i in range(n)]
    # c [items, remaining capacity]
    # important: this code assumes strictly positive objective values
    for i in range(n):
        for j in range(capacity+1):
            if (weights[i] > j):
                c[i][j] = c[i-1][j]
            else:
                c_add = obj[i] + c[i-1][j-weights[i]]
                if c_add > c[i-1][j]:
                    c[i][j] = c_add
                    added[i][j] = True
                else:
                    c[i][j] = c[i-1][j]

    # backtrack to find solution
    i = n-1
    j = capacity

    solution = []
    while i >= 0 and j >= 0:
        if added[i][j]:
            solution.append(i)
            j -= weights[i]
        i -= 1
        
    return c[n-1][capacity], solution

prob.relaxed_solver = solve_subproblem

dippy.Solve(prob, {
    'TolZero': '%s' % tol,
    'doCut': '1',
#    'logLevel': '3', 
})

for m in MACHINES:
    print 
    print "Machine %d assigned tasks" %m,
    for t in TASKS:
        v = assignVars[m][t].varValue
        if v:
            print "%d" %t,
            
if prob.display_mode != 'off':
    if (prob.Tree.attr['display'] == 'pygame') or (prob.Tree.attr['display'] == 'xdot'):
        prob.Tree.display()




#!/usr/bin/python

import commands
import os
from math import *
from   optparse import OptionParser
from xml.dom.minidom import parse
from random import *
import sys

def generate_uniform(n, min_v, max_v, seed):
    weights = []
    r = Random()
    for i in range(n):
	weights.append(r.uniform(min_v, max_v))
    return weights

def generate_linear(n, min_v, max_v):
    weights = []
    delta = max_v - min_v
    for i in range(n):
	weights.append(min_v + (delta*(i)) / float(n))
    return weights

def generate_fixed(value_sting):
    weights = []
    values = value_string.split()
    for value in values:
	weights.append(float(value))
    return weights


def generate_inv_strongly_correlated(profits, delta):
    weights = []
#    print profits
    max_profit = max(profits)
    for i in range(len(profits)):
	weights.append(max_profit - profits[i]/delta )
    return weights

def generate_inv_weakly_correlated(profits, delta):
    weights = []
    max_profit = max(profits)
    r = Random()

    for i in range(len(profits)):
	weights.append( r.uniform(0, max_profit- profits[i]/delta))
    return weights


def generate_strongly_correlated(profits, delta):
    weights = []
#    print profits
    max_profit = max(profits)
    for i in range(len(profits)):
	weights.append(profits[i] + max_profit/delta )
    return weights

def generate_weakly_correlated(profits, delta):
    weights = []
    max_profit = max(profits)
    r = Random()

    for i in range(len(profits)):
	weights.append( r.uniform(max(0, profits[i] - max_profit/delta), profits[i] + max_profit/delta))
    return weights

#####################################################
## Parse the command line options.
parser = OptionParser()
parser.add_option("--def_file", dest="def_file", type="string", default="",
		help="instance definition file" )


(options, args) = parser.parse_args()
out_file = sys.stdout
# Construct xml parser and read the definition file. 
dom = parse(options.def_file)
mmkps = dom.getElementsByTagName("mmkp")

for mmkp in mmkps:
#{
    assert mmkp.attributes.has_key('name')
    assert mmkp.attributes.has_key('classes')
    assert mmkp.attributes.has_key('dimensions')
    assert mmkp.attributes.has_key('instances')

    serie_name = mmkp.attributes['name'].value    
    num_classes = int(mmkp.attributes['classes'].value)
    num_dimensions= int(mmkp.attributes['dimensions'].value)
    num_instances = int(mmkp.attributes['instances'].value) 
    num_constraint_levels = int(mmkp.attributes['constraint_levels'].value) 

    if mmkp.attributes.has_key('p_value'):
	p_value = mmkp.attributes['p_value'].value
    else:
	p_value = "float" 
    
    if mmkp.attributes.has_key('w_value'):
	w_value = mmkp.attributes['w_value'].value
    else:
	w_value = "float" 

    if mmkp.attributes.has_key('c_value'):
	c_value = mmkp.attributes['c_value'].value
    else:
	c_value = "float" 

    # inst_type could be "generic" or "wsn"
    if mmkp.attributes.has_key('inst_type'):
	inst_type = mmkp.attributes['inst_type'].value
    else:
	inst_type = "generic" 
    
    if inst_type != "generic":
	print "Cannot process definition files for non-generic instances."
	exit()

    for inst_id in range(num_instances):
    #{
	
	instance_txt = [] 
	instance_txt.append("# MMKP instances generated from definition file: " + options.def_file + '\n')
        instance_txt.append("# serie_name\t" + serie_name + "\n")
	instance_txt.append("# num_instances\t" + repr(num_instances) +"\n")

	instance_txt.append('instance_id\t' +repr(inst_id+1) + '\n')
	instance_txt.append('num_class\t' +repr(num_classes)+ '\n')
	instance_txt.append('num_dimension\t' +repr(num_dimensions)+ '\n') 
	
	# Process each class in an mmkp
	classes = mmkp.getElementsByTagName('class')
	sum_min_weight = []
	sum_max_weight = []
	for d in range(num_dimensions):
	    sum_min_weight.append(0)
	    sum_max_weight.append(0)
	
	c_id = 0
	for one_class in classes: 
	#{
	    num_item = int(one_class.attributes['items'].value) 
	    
	    if one_class.attributes.has_key('repeat'):
		c_repeat = int(one_class.attributes['repeat'].value)  
	    else: 
		c_repeat = 1
	   
	    for repeat_id in range(c_repeat):
	    #{
		c_id = c_id + 1
		instance_txt.append('# class_id\t' + repr(c_id) + '\n')
		instance_txt.append('num_item\t' + repr(num_item)+ '\n')
		# Process profits
		profit_function = one_class.getElementsByTagName('profit')[0]
		method = profit_function.attributes['method'].value
		if method == 'uniform':
		#{
		    parameters = profit_function.getElementsByTagName('parameters')[0]
		    min_v = float(parameters.attributes['min'].value)
		    max_v = float(parameters.attributes['max'].value)
		    profit_values = generate_uniform(num_item, min_v, max_v, c_id)
		#}
		elif method == 'linear':
		#{
		    parameters = profit_function.getElementsByTagName('parameters')[0]
		    min_v = float(parameters.attributes['min'].value)
		    max_v = float(parameters.attributes['max'].value)
		    profit_values = generate_linear(num_item, min_v, max_v)
		#}
		elif method == 'fixed':
		#{
		    parameters = profit_function.getElementsByTagName('parameters')[0]
		    value_string= (parameters.attributes['values'].value)
		    profit_values = generate_fixed(value_string)
		#}

		# Process each type of weight
		weight_functions = one_class.getElementsByTagName('weight')	
		weight_value_matrix=[]
		
		w_id = 0
		for weight_function in weight_functions:
		#{
		    method = weight_function.attributes['method'].value
		    parameters = weight_function.getElementsByTagName('parameters')[0]
		    if weight_function.attributes.has_key('repeat'):
			w_repeat = int(weight_function.attributes['repeat'].value)  
		    else: 
			w_repeat = 1
		   
		    for weight_repeat_id in range(w_repeat):
		    #{

			w_id = w_id + 1
			if method == 'uniform':
			    min_v = float(parameters.attributes['min'].value)
			    max_v = float(parameters.attributes['max'].value)
			    weight_values = generate_uniform(num_item, min_v, max_v, c_id+w_id)

			elif method == 'strongly_correlated':
			    delta = float(parameters.attributes['delta'].value)
			    weight_values = generate_strongly_correlated(profit_values, delta)

			elif method == 'weakly_correlated':
			    delta = float(parameters.attributes['delta'].value)
			    weight_values = generate_weakly_correlated(profit_values, delta)
		    	elif method == 'inv_strongly_correlated':
			    delta = float(parameters.attributes['delta'].value)
			    weight_values = generate_inv_strongly_correlated(profit_values, delta)
			elif method == 'inv_weakly_correlated':
			    delta = float(parameters.attributes['delta'].value)
			    weight_values = generate_inv_weakly_correlated(profit_values, delta)

			weight_value_matrix.append(weight_values)
			sum_min_weight[w_id-1] = sum_min_weight[w_id-1] + min(weight_values)
			sum_max_weight[w_id-1] = sum_max_weight[w_id-1] + max(weight_values)
#		print sum_min_weight, min(weight_values)
#		print sum_max_weight, max(weight_values)
#		print weight_values
		    #}
		#}
		
		for i in range(len(profit_values)):
		    if p_value == "int":
			instance_txt.append("%d\t" % ceil(profit_values[i]))
		    else:
			instance_txt.append("%.2f\t" % profit_values[i])
		    for j in range(num_dimensions):
			if w_value == "int":    
			    instance_txt.append("%d\t" % ceil(weight_value_matrix[j][i]))
			else: 
			    instance_txt.append("%.2f\t" % weight_value_matrix[j][i])
		    instance_txt.append('\n')

	    #}
	#}
	
	constraint_txt = []
	for level in range(num_constraint_levels):
	#{
	    out_file_name = serie_name + '_' + repr(level) +'_' + repr(inst_id) + '.data'
	    out_file = open(out_file_name, "w")
	    for line in instance_txt:
		out_file.write(line)
	    
	    out_file.write("c:\t")
	    for d in range(num_dimensions):
		constraint = sum_min_weight[d] + (float(level+1)/float(num_constraint_levels+1)) * (sum_max_weight[d] - sum_min_weight[d])
		if c_value == "int":
		    out_file.write("%d\t" % ceil(constraint))
		else: 
		    out_file.write("%.2f\t" % constraint)
	    out_file.write('\n')
	    out_file.close()
	#}	
    #}
    
#}



#!/usr/bin/env python

from bin_pack_func import BinPackProb, formulate, solve

if __name__ == '__main__':
    # Python starts here
    bpp = BinPackProb(ITEMS  = [1, 2, 3, 4, 5, 6],
                      volume = {1: 2, 2: 5, 3: 3, 4: 3, 5: 3, 6: 2},
                      capacity = 9)
  
    prob = formulate(bpp)
  
    prob.tol = pow(pow(2, -24), 2.0 / 3.0)
    xopt = solve(prob)
  
    if xopt is not None:
        for var in prob.variables():
            print var.name, "=", xopt[var]
    else:
        print "Dippy could not find and optimal solution"
  
    if prob.display_mode != 'off':
        numNodes = len(prob.Tree.get_node_list())
        if (prob.Tree.attr['display'] == 'pygame') or (prob.Tree.attr['display'] == 'xdot'):
            prob.Tree.display()



from random import randint

# The requirements for the products
REQUIREMENT = {
    1 : 7,
    2 : 5,
    3 : 3,
    4 : 2,
    5 : 2
}

# Set of all products
PRODUCTS = REQUIREMENT.keys()
PRODUCTS.sort()

# Costs of the facilities
FIXED_COST = {
    1 : 10,
    2 : 20,
    3 : 16, 
    4 : 1, 
    5 : 2
}

# Set of facilities
LOCATIONS = FIXED_COST.keys()
LOCATIONS.sort()

# The capacity of the facilities

CAPACITY = 8



#!/usr/bin/env python

DEBUGGING = False

import sys

from pulp import *

try:
    import path
except ImportError:
    pass
        
if DEBUGGING:
    import dippy
else:
    import coinor.dippy as dippy

from math import floor, ceil

tol = pow(pow(2, -24), 2.0 / 3.0)

from facility_ex2 import REQUIREMENT, PRODUCTS
from facility_ex2 import LOCATIONS, CAPACITY
try:
    from facility_ex2 import FIXED_COST
except ImportError:
    FIXED_COST = [1 for i in LOCATIONS]

try:
    from facility_ex2 import ASSIGNMENTS
except ImportError:
    ASSIGNMENTS = [(i, j) for i in LOCATIONS for j in PRODUCTS]

try:
    from facility_ex2 import ASSIGNMENT_COSTS
except ImportError:
    ASSIGNMENT_COSTS = dict((i, 0) for i in ASSIGNMENTS)

display_mode = 'xdot'

prob = dippy.DipProblem("Facility Location", display_mode = display_mode,
                        layout = 'dot', display_interval = 0)

assign_vars = LpVariable.dicts("x", ASSIGNMENTS, 0, 1, LpBinary)
use_vars    = LpVariable.dicts("y", LOCATIONS, 0, 1, LpBinary)

debug_print = False

debug_print_lp = False

prob += (lpSum(use_vars[i] * FIXED_COST[i] for i in LOCATIONS) +
         lpSum(assign_vars[j] * ASSIGNMENT_COSTS[j] for j in ASSIGNMENTS), 
         "min")

# assignment constraints
for j in PRODUCTS:
    prob += lpSum(assign_vars[(i, j)] for i in LOCATIONS) == 1

# Aggregate capacity constraints
for i in LOCATIONS:
    prob.relaxation[i] += lpSum(assign_vars[(i, j)] * REQUIREMENT[j]
                                for j in PRODUCTS) <= CAPACITY * use_vars[i]

# Disaggregate capacity constraints
for i, j in ASSIGNMENTS:
    prob.relaxation[i] += assign_vars[(i, j)] <= use_vars[i]

def solve_subproblem(prob, key, redCosts):
    if debug_print:
        print "solve_subproblem..."
        print redCosts
   
    loc = key

    # Calculate effective objective coefficient of products

    avars = [assign_vars[(loc, j)] for j in PRODUCTS]
    obj = [max(-redCosts[assign_vars[(loc, j)]], 0) for j in PRODUCTS]
    weights = [REQUIREMENT[j] for j in PRODUCTS]
   
    # Use 0-1 KP to max. total effective value of products at location
    z, solution = knapsack01(obj, weights, CAPACITY)
   
    # Get the reduced cost of the knapsack solution
    if debug_print:
        print [(v, redCosts[v]) for v in avars]
        print obj
        print "z, solution =", z, solution
        print "redCosts[use_vars[loc]] =", redCosts[use_vars[loc]]

    rc = redCosts[use_vars[loc]] - z

    if debug_print:
        print "Fixed cost, rc", FIXED_COST[loc], rc

    if rc > tol: # ... or an empty location is "useful"
       
        var_values = {}

        var_tuple = (0.0, 0.0, var_values)
        if debug_print:
            print "Zero solution is optimal"
            print var_tuple
        return [var_tuple]

    var_values = dict([(avars[i], 1) for i in solution])
    var_values[use_vars[loc]] = 1

    cost = FIXED_COST[loc] + sum([ASSIGNMENT_COSTS[(loc, PRODUCTS[j])] for j in solution])    
    var_tuple = (cost, rc, var_values)
    rcCheck = 0.0
    for v in var_values.keys():
        rcCheck += redCosts[v] * var_values[v]
    if debug_print:
        print "Checking rc calc", rc, rcCheck 
        print var_tuple
    return [var_tuple]


def knapsack01(obj, weights, capacity):
    """ 0/1 knapsack solver, maximizes profit. weights and capacity integer """
        
    debug_subproblem = False
    
    assert len(obj) == len(weights)
    n = len(obj)
    if n == 0:
        return 0, []

    if (debug_subproblem):
        relaxation = pulp.LpProblem('relaxation', pulp.LpMaximize)
        relax_vars = [str(i) for i in range(n)]
        var_dict   = LpVariable.dicts("", relax_vars, 0, 1, LpBinary)
        relaxation += lpSum(var_dict[str(i)] * weights[i] for i in range(n)) <= capacity
        relaxation += lpSum(var_dict[str(i)] * obj[i] for i in range(n))
        relaxation.solve()
        relax_obj = value(relaxation.objective)

        solution =  [i for i in range(n) if var_dict[str(i)].varValue > tol ]

        print relax_obj, solution


    c = [[0]*(capacity+1) for i in range(n)]
    added = [[False]*(capacity+1) for i in range(n)]
    # c [items, remaining capacity]
    # important: this code assumes strictly positive objective values
    for i in range(n):
        for j in range(capacity+1):
            if (weights[i] > j):
                c[i][j] = c[i-1][j]
            else:
                c_add = obj[i] + c[i-1][j-weights[i]]
                if c_add > c[i-1][j]:
                    c[i][j] = c_add
                    added[i][j] = True
                else:
                    c[i][j] = c[i-1][j]

    # backtrack to find solution
    i = n-1
    j = capacity

    solution = []
    while i >= 0 and j >= 0:
        if added[i][j]:
            solution.append(i)
            j -= weights[i]
        i -= 1
        
    return c[n-1][capacity], solution

def generate_weight_cuts(prob, sol):
    # Define mu and T for each knapsack
    mu = {}
    S = {}
    for i in LOCATIONS:
        mu[i] = CAPACITY
        S[i] = []
    
    # Use current assign_var values to assign items to locations
    assigning = True 
    while assigning:
        bestValue = 0
        bestAssign = None
        for i in LOCATIONS:
            for j in PRODUCTS:
                if j not in S[i]: # If this product is not in the subset
                    if (sol[assign_vars[(i, j)]] > bestValue) \
                    and (REQUIREMENT[j] <= mu[i]):
                        # The assignment variable for this product is closer
                        # to 1 than any other product checked, and "fits" in
                        # this location's remaining space
                        bestValue = sol[assign_vars[(i, j)]]
                        bestAssign = (i, j)
        # Make the best assignment found across all products and locactions
        if bestAssign:
            (i,j) = bestAssign
            mu[i] -= REQUIREMENT[j] # Decrease spare CAPACITY at this location
            S[i].append(j) # Assign this product to this location's set
        else:
            assigning = False # Didn't find anything to assign - stop

    # Generate the weight cuts from the sets found above
    new_cuts = []
    for i in LOCATIONS:
        if len(S[i]) > 0: # If an item assigned to this location
            con = LpAffineExpression() # Start a new constraint
            con += sum(REQUIREMENT[j] * assign_vars[(i, j)] 
                            for j in S[i])
            con += sum(max(0, REQUIREMENT[j] - mu[i]) *
                            assign_vars[(i, j)] for j in PRODUCTS 
                            if j not in S[i])
            new_cuts.append(con <= CAPACITY - mu[i])

    # Return the set of cuts we created to DIP
    if len(new_cuts) > 0:
        return new_cuts

def first_fit_heuristic():
    # Sort the items in descending weight order
    productReqs = [(REQUIREMENT[j],j) for j in PRODUCTS]
    productReqs.sort(reverse=True)

    # Add items to locations, fitting in as much
    # as possible at each location.
    allLocations = []
    while len(productReqs) > 0:
        waste = CAPACITY
        currentLocation = []
        j = 0
        while j < len(productReqs):
            # Can we fit this product?
            if productReqs[j][0] <= waste:
                currentLocation.append(productReqs[j][1]) # index
                waste -= productReqs[j][0] # requirement
                productReqs.pop(j)
            else:
                # Try to fit next item
                j += 1
        allLocations.append((currentLocation, waste))

    # Return a list of tuples: ([products],waste)
    return allLocations

def first_fit():
    # Use generic first-fit heuristic that is shared
    # between heuristics and initial variable generation
    allLocations = first_fit_heuristic()

    # Convert to decision variable values
    sol = {}
    for i in LOCATIONS:
        for j in PRODUCTS:
            sol[assign_vars[(i, j)]] = 0
        sol[use_vars[i]] = 0

    index = 0
    for loc in allLocations:
        i = LOCATIONS[index]
        sol[use_vars[i]] = 1
        for j in loc[0]:
            sol[assign_vars[(i, j)]] = 1
        index += 1
        
    return sol

def frac_fit(xhat):
    # Initialise solution
    sol = {}
    waste = {}
    for i in LOCATIONS:
        for j in PRODUCTS: sol[assign_vars[(i, j)]] = 0
        sol[use_vars[i]] = 0
        waste[i] = 0
        
    # Get the list of non-zero fractional assignments
    fracAssigns = [ (xhat[assign_vars[(i, j)]], (i, j))
                     for i in LOCATIONS for j in PRODUCTS
                     if xhat[assign_vars[(i, j)]] > tol ]
    fracAssigns.sort()

    # Track which products and locations have been used
    notAllocated = dict((j,True) for j in PRODUCTS)
    notUsed      = dict((i,True) for i in LOCATIONS)
    while len(fracAssigns) > 0:
        fracX = fracAssigns.pop() # Get best frac. assignment left
        (i,j) = fracX[1]
        if notAllocated[j]:
            if notUsed[i]: # Create a new location if needed
                notUsed[i] = False
                sol[use_vars[i]] = 1
                waste[i] = CAPACITY
            if REQUIREMENT[j] <= waste[i]: # Space left?
                sol[assign_vars[(i, j)]] = 1
                notAllocated[j] = False
                waste[i] -= REQUIREMENT[j]
    
    # Allocate the remaining products
    unallocated = [(REQUIREMENT[j],j) for j in PRODUCTS
                                      if notAllocated[j]]
    unallocated.sort(reverse=True)
    unused = [i for i in LOCATIONS if notUsed[i]]
    while len(unallocated) > 0:
        waste = CAPACITY
        index = 0
        loc = unused.pop()
        while index < len(unallocated):
            (j_req, j) = unallocated[index]
            if j_req <= waste:
                unallocated.pop(index)
                sol[assign_vars[(loc, j)]] = 1
                waste -= j_req
            else: index += 1
        sol[use_vars[loc]] = 1

    return sol

def heuristics(prob, xhat, cost):
    sols = []
    if prob.root_heuristic:
        prob.root_heuristic = False # Don't run twice
        sol = first_fit()
        sols.append(sol)
    if prob.node_heuristic:
        sol = frac_fit(xhat)
        sols.append(sol)

    if len(sols) > 0:
        return sols
      
def init_first_fit(prob):

    locations = first_fit_heuristic()
    bvs = []
    index = 0
    for loc in locations:
        i = LOCATIONS[index]
        if debug_print:
            print [(assign_vars[(i, j)], 1) for j in loc[0]]
        var_values = dict([(assign_vars[(i, j)], 1) for j in loc[0]])
        var_values[use_vars[i]] = 1
        dv = (loc[1], var_values)
        bvs.append((i, dv))
        index += 1
    print bvs
    return bvs

def init_one_each(prob):
    bvs = []
    if debug_print:
        print "LOCATIONS =", LOCATIONS
    for index, loc in enumerate(LOCATIONS):
        lc = [PRODUCTS[index]]
        waste = CAPACITY - REQUIREMENT[PRODUCTS[index]]
        var_values = dict([(assign_vars[(loc, j)], 1) for j in lc])
        var_values[use_vars[loc]] = 1

        dv = (waste, var_values)
        bvs.append((loc, dv))
        if debug_print:
            print bvs
    return bvs

if debug_print_lp:
    prob.writeLP('facility_main.lp')
    for n, i in enumerate(LOCATIONS):
        prob.writeRelaxed(n, 'facility_relax%s.lp' % i);

prob.writeFull('facility.lp', 'facility.dec')

#prob.branch_method = choose_antisymmetry_branch
#prob.relaxed_solver = solve_subproblem
#prob.init_vars = init_one_each
#prob.init_vars = init_first_fit
#prob.generate_cuts = generate_weight_cuts
#prob.heuristics = heuristics
#prob.root_heuristic = True
#prob.node_heuristic = True

dippy.Solve(prob, {
    'TolZero': '%s' % tol,
    'doPriceCut': '1',
    'CutCGL': '0',
#    'SolveMasterAsIp': '0',
#    'generateInitVars': '1',
#    'LogDebugLevel': 3,
#    'LogDumpModel': 5,
#    'ALPS' :
#    {'msgLevel' : 3}
})

if prob.display_mode != 'off':
    numNodes = len(prob.Tree.get_node_list())
    if prob.Tree.attr['display'] == 'svg':
        prob.Tree.write_as_svg(filename = "facility_node%d" % (numNodes + 1), 
                               prevfile = "facility_node%d" % numNodes)
    prob.Tree.display()

# print solution
print "Optimal solution found!" 
print "************************************"
for i in LOCATIONS:
    if use_vars[i].varValue > 0:
        print "Location ", i, " is assigned: ",
        print [j for j in PRODUCTS if assign_vars[(i, j)].varValue > 0]
print "************************************"
print
